@@ -15,8 +15,8 @@
  * 
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 021110-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301 USA.
  *
  * --
  * Pass 2 iterates through the directory blocks that pass 1 found under 
@@ -43,7 +43,7 @@
 #include "fsck.h"
 #include "pass2.h"
 #include "problem.h"
-#include "strings.h"
+#include "o2fsck_strings.h"
 #include "util.h"
 
 static const char *whoami = "pass2";
@@ -613,7 +613,7 @@ static errcode_t fix_dirent_dups(o2fsck_state *ost,
 
 	if (was_set) {
 		printf("Directory inode %"PRIu64" contains a duplicate "
-		       "occurance " "of the file name '%.*s' but fsck was "
+		       "occurrence " "of the file name '%.*s' but fsck was "
 		       "unable to come up with a unique name so this duplicate "
 		       "name will not be dealt with.\n.",
 			dbe->e_ino, dirent->name_len, dirent->name);
@@ -621,7 +621,7 @@ static errcode_t fix_dirent_dups(o2fsck_state *ost,
 	}
 
 	if (!prompt(ost, PY, PR_DIRENT_DUPLICATE,
-		    "Directory inode %"PRIu64" contains a duplicate occurance "
+		    "Directory inode %"PRIu64" contains a duplicate occurrence "
 		    "of the file name '%.*s'.  Replace this duplicate name "
 		    "with '%s'?", dbe->e_ino, dirent->name_len, dirent->name,
 		    new_name)) {
