@@ -18,8 +18,8 @@
  * 
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 021110-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301 USA.
  *
  * Authors: Mark Fasheh <mark.fasheh@oracle.com>
  */
@@ -803,7 +803,7 @@ static errcode_t load_fsdlm(struct o2dlm_ctxt *ctxt)
 		goto out;
 	}
 
-	ctxt->ct_lib_handle = dlopen("libdlm_lt.so",
+	ctxt->ct_lib_handle = dlopen("libdlm_lt.so.3",
 				     RTLD_NOW | RTLD_LOCAL);
 	if (!ctxt->ct_lib_handle)
 		goto out;
